/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jbi.messaging.exchange;

import static org.junit.Assert.assertEquals;

import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfOperation.MEPPatternConstants;
import org.ow2.petals.commons.log.PetalsExecutionContext;
import org.ow2.petals.jbi.messaging.PetalsDeliveryChannel;
import org.ow2.petals.jbi.messaging.exchange.impl.DefaultMessageExchangeFactory;
import org.ow2.petals.jbi.servicedesc.endpoint.Location;

/**
 * @author Olivier Fabre - EBM WebSourcing
 */
public class MessageExchangeFactoryImplTest extends MessageExchangeCheckHelper {

    private Location consumerLocationMock = Mockito.mock(Location.class);

    private PetalsDeliveryChannel channel;

    private Logger logger = Logger.getAnonymousLogger();

    @Before
    public void before() {
        this.channel = Mockito.mock(PetalsDeliveryChannel.class);
        final ComponentContext mockCC = Mockito.mock(ComponentContext.class);
        Mockito.when(this.channel.getContext()).thenReturn(mockCC);
        Mockito.when(this.channel.getLogger()).thenReturn(this.logger);
        Mockito.when(this.channel.getLocation()).thenReturn(this.consumerLocationMock);
        Mockito.when(this.channel.isOpened()).thenReturn(true);
    }

    @After
    public void after() {
        PetalsExecutionContext.clearAll();
    }

    @Test
    public void testBuildUseBuilderConsumerEp() throws MessagingException {
        AbstractMessageExchangeFactory abstractMessageExchangeFactory = (AbstractMessageExchangeFactory) DefaultMessageExchangeFactory
                .create(this.channel);
        PetalsMessageExchange messageExchangeImpl = abstractMessageExchangeFactory
                .buildExchange(MEPPatternConstants.IN_ONLY.value());

        assertEquals(consumerLocationMock, messageExchangeImpl.getConsumerLocation());
    }
    
    @Test
    public void testBuildUseBuilderConsumerEpForEndpoint() throws MessagingException {
        ServiceEndpoint targetEndpointMock = Mockito.mock(ServiceEndpoint.class);

        AbstractMessageExchangeFactory abstractMessageExchangeFactory = (AbstractMessageExchangeFactory) DefaultMessageExchangeFactory
                .createForEndpoint(this.channel, targetEndpointMock);
        PetalsMessageExchange messageExchangeImpl = abstractMessageExchangeFactory
                .buildExchange(MEPPatternConstants.IN_ONLY.value());

        assertEquals(consumerLocationMock, messageExchangeImpl.getConsumerLocation());
        assertEquals(targetEndpointMock, messageExchangeImpl.getEndpoint());
        assertIdIsPetalsQualifiedUUID(messageExchangeImpl.getExchangeId());
    }
    
    @Test
    public void testBuildUseBuilderConsumerEpForInterface() throws MessagingException {
        QName targetInterface = new QName("http://test","interface");
        AbstractMessageExchangeFactory abstractMessageExchangeFactory = (AbstractMessageExchangeFactory) DefaultMessageExchangeFactory
                .createForInterface(this.channel, targetInterface);
        PetalsMessageExchange messageExchangeImpl = abstractMessageExchangeFactory
                .buildExchange(MEPPatternConstants.IN_ONLY.value());

        assertEquals(consumerLocationMock, messageExchangeImpl.getConsumerLocation());
        assertEquals(targetInterface, messageExchangeImpl.getInterfaceName());
    }
    
    @Test
    public void testBuildUseBuilderConsumerEpForService() throws MessagingException {
        QName targetService = new QName("http://test","service");
        AbstractMessageExchangeFactory abstractMessageExchangeFactory = (AbstractMessageExchangeFactory) DefaultMessageExchangeFactory
                .createForService(this.channel, targetService);
        PetalsMessageExchange messageExchangeImpl = abstractMessageExchangeFactory
                .buildExchange(MEPPatternConstants.IN_ONLY.value());

        assertEquals(consumerLocationMock, messageExchangeImpl.getConsumerLocation());
        assertEquals(targetService, messageExchangeImpl.getService());
    }
}
