/**
 * Copyright (c) 2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jbi.messaging.exchange;

import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Arrays;
import java.util.Collection;

import javax.xml.stream.XMLInputFactory;
import javax.xml.transform.Source;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stax.StAXSource;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.io.IOUtils;
import org.custommonkey.xmlunit.Diff;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;
import org.ow2.petals.jbi.messaging.exchange.impl.NormalizedMessageImpl;
import org.ow2.petals.jbi.xml.BytesSource;
import org.xml.sax.InputSource;

import com.ebmwebsourcing.easycommons.xml.DOMHelper;
import com.ebmwebsourcing.easycommons.xml.SourceHelper;

@RunWith(Parameterized.class)
public class NormalizedMessageImplContentTest {

    public static final String CONTROL_CONTENT;

    static {
        try {
            CONTROL_CONTENT = IOUtils.toString(getTestContent());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Parameters
    public static Collection<Object[]> data() {
        XMLInputFactory factory = XMLInputFactory.newInstance();
        try {
            return Arrays.asList(new Object[][] { { new BytesSource(IOUtils.toByteArray(getTestContent())) },
                    { DOMHelper.parseAsDOMSource(getTestContent()) },
                    { new SAXSource(new InputSource(getTestContent().openStream())) },
                    { new SAXSource(new InputSource(new InputStreamReader(getTestContent().openStream()))) },
                    { new SAXSource(new InputSource(getTestContent().toString())) },
                    { new StAXSource(factory.createXMLStreamReader(getTestContent().openStream())) },
                    { new StAXSource(factory.createXMLEventReader(getTestContent().openStream())) },
                    { new StreamSource(new File(getTestContent().toURI())) },
                    { new StreamSource(getTestContent().openStream()) },
                    { new StreamSource(new InputStreamReader(getTestContent().openStream())) },
                    { new StreamSource(getTestContent().toString()) }, });
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static URL getTestContent() {
        return Thread.currentThread().getContextClassLoader().getResource("test-content.xml");
    }

    @Parameter
    public Source source;

    @Test
    public void setReRedeable() throws Exception {
        NormalizedMessageImpl msg = new NormalizedMessageImpl();
        msg.setContent(source);

        Diff once = new Diff(CONTROL_CONTENT, SourceHelper.toString(msg.getContent()));
        assertTrue(once.toString(), once.identical());
        Diff twice = new Diff(CONTROL_CONTENT, SourceHelper.toString(msg.getContent()));
        assertTrue(twice.toString(), twice.identical());
    }
}
